@@ -1,6 +1,8 @@
-use super::kernel_error::{self, IntoResult, KernelError};
-use super::proc_maps::{DyldInfo, DyldInfoManager, Modification};
-use super::thread_profiler::ThreadProfiler;
+use crate::error::SamplingError;
+use crate::kernel_error::{IntoResult, KernelError};
+use crate::proc_maps::{DyldInfo, DyldInfoManager, Modification};
+use crate::thread_profiler::ThreadProfiler;
+use gecko_profile::debugid::DebugId;
 use mach::mach_types::thread_act_port_array_t;
 use mach::mach_types::thread_act_t;
 use mach::message::mach_msg_type_number_t;
@@ -13,15 +15,16 @@ use std::collections::hash_map::Entry;
 use std::collections::{HashMap, HashSet};
 use std::mem;
 use std::path::Path;
-use std::time::{Duration, Instant};
+use std::time::{Duration, Instant, SystemTime};
 
-use super::gecko_profile::ProfileBuilder;
+use gecko_profile::ProfileBuilder;
 
 pub struct TaskProfiler {
     task: mach_port_t,
     pid: u32,
     interval: Duration,
     start_time: Instant,
+    start_time_system: SystemTime,
     end_time: Option<Instant>,
     live_threads: HashMap<thread_act_t, ThreadProfiler>,
     dead_threads: Vec<ThreadProfiler>,
@@ -29,6 +32,7 @@ pub struct TaskProfiler {
     libs: Vec<DyldInfo>,
     executable_lib: Option<DyldInfo>,
     command_name: String,
+    ignored_errors: Vec<SamplingError>,
 }
 
 impl TaskProfiler {
@@ -36,23 +40,25 @@ impl TaskProfiler {
         task: mach_port_t,
         pid: u32,
         now: Instant,
+        now_system: SystemTime,
         command_name: &str,
         interval: Duration,
-    ) -> kernel_error::Result<Self> {
-        let thread_acts = get_thread_list(task)?;
+    ) -> Option<Self> {
+        let thread_acts = get_thread_list(task).ok()?;
         let mut live_threads = HashMap::new();
         for (i, thread_act) in thread_acts.into_iter().enumerate() {
             // Pretend that the first thread is the main thread. Might not be true.
             let is_main = i == 0;
-            if let Some(thread) = ThreadProfiler::new(task, pid, now, thread_act, now, is_main)? {
+            if let Some(thread) = ThreadProfiler::new(task, pid, thread_act, now, is_main) {
                 live_threads.insert(thread_act, thread);
             }
         }
-        Ok(TaskProfiler {
+        Some(TaskProfiler {
             task,
             pid,
             interval,
             start_time: now,
+            start_time_system: now_system,
             end_time: None,
             live_threads,
             dead_threads: Vec::new(),
@@ -60,20 +66,34 @@ impl TaskProfiler {
             libs: Vec::new(),
             command_name: command_name.to_owned(),
             executable_lib: None,
+            ignored_errors: Vec::new(),
         })
     }
 
-    pub fn sample(&mut self, now: Instant) -> kernel_error::Result<bool> {
+    pub fn sample(&mut self, now: Instant) -> Result<bool, SamplingError> {
         let result = self.sample_impl(now);
         match result {
             Ok(()) => Ok(true),
-            Err(KernelError::MachSendInvalidDest) => Ok(false),
-            Err(KernelError::Terminated) => Ok(false),
+            Err(SamplingError::ProcessTerminated(_, _)) => Ok(false),
+            Err(err @ SamplingError::Ignorable(_, _)) => {
+                self.ignored_errors.push(err);
+                if self.ignored_errors.len() >= 10 {
+                    println!(
+                        "Treating process \"{}\" [pid: {}] as terminated after 10 unknown errors:",
+                        self.command_name, self.pid
+                    );
+                    println!("{:#?}", self.ignored_errors);
+                    Ok(false)
+                } else {
+                    // Pretend that sampling worked and that the thread is still alive.
+                    Ok(true)
+                }
+            }
             Err(err) => Err(err),
         }
     }
 
-    fn sample_impl(&mut self, now: Instant) -> kernel_error::Result<()> {
+    fn sample_impl(&mut self, now: Instant) -> Result<(), SamplingError> {
         // First, check for any newly-loaded libraries.
         let changes = self
             .lib_info_manager
@@ -95,10 +115,7 @@ impl TaskProfiler {
         }
 
         // Enumerate threads.
-        let thread_acts = get_thread_list(self.task).map_err(|err| match err {
-            KernelError::InvalidArgument => KernelError::Terminated,
-            err => err,
-        })?;
+        let thread_acts = get_thread_list(self.task)?;
         let previously_live_threads: HashSet<_> =
             self.live_threads.iter().map(|(t, _)| *t).collect();
         let mut now_live_threads = HashSet::new();
@@ -107,16 +124,12 @@ impl TaskProfiler {
             let thread = match entry {
                 Entry::Occupied(ref mut entry) => entry.get_mut(),
                 Entry::Vacant(entry) => {
-                    match ThreadProfiler::new(
-                        self.task,
-                        self.pid,
-                        self.start_time,
-                        thread_act,
-                        now,
-                        false,
-                    )? {
-                        Some(thread) => entry.insert(thread),
-                        None => continue,
+                    if let Some(thread) =
+                        ThreadProfiler::new(self.task, self.pid, thread_act, now, false)
+                    {
+                        entry.insert(thread)
+                    } else {
+                        continue;
                     }
                 }
             };
@@ -157,8 +170,13 @@ impl TaskProfiler {
             })
             .unwrap_or(self.command_name);
 
-        let mut profile_builder =
-            ProfileBuilder::new(self.start_time, &name, self.pid, self.interval);
+        let mut profile_builder = ProfileBuilder::new(
+            self.start_time,
+            self.start_time_system,
+            &name,
+            self.pid,
+            self.interval,
+        );
         let all_threads = self
             .live_threads
             .into_iter()
@@ -170,7 +188,7 @@ impl TaskProfiler {
         }
 
         if let Some(end_time) = self.end_time {
-            profile_builder.set_end_time(end_time.duration_since(self.start_time));
+            profile_builder.set_end_time(end_time);
         }
 
         for DyldInfo {
@@ -186,9 +204,17 @@ impl TaskProfiler {
                 (Some(uuid), Some(arch)) => (uuid, arch),
                 _ => continue,
             };
-            let name = Path::new(&file).file_name().unwrap().to_str().unwrap();
+            let path = Path::new(&file);
             let address_range = address..(address + vmsize);
-            profile_builder.add_lib(name, &file, &uuid, arch, &address_range);
+            profile_builder.add_lib(
+                path,
+                None,
+                path,
+                DebugId::from_uuid(uuid),
+                Some(arch),
+                address,
+                address_range,
+            );
         }
 
         for subtask in subtasks {
@@ -199,10 +225,19 @@ impl TaskProfiler {
     }
 }
 
-fn get_thread_list(task: mach_port_t) -> kernel_error::Result<Vec<thread_act_t>> {
+fn get_thread_list(task: mach_port_t) -> Result<Vec<thread_act_t>, SamplingError> {
     let mut thread_list: thread_act_port_array_t = std::ptr::null_mut();
     let mut thread_count: mach_msg_type_number_t = Default::default();
-    unsafe { task_threads(task, &mut thread_list, &mut thread_count) }.into_result()?;
+    unsafe { task_threads(task, &mut thread_list, &mut thread_count) }
+        .into_result()
+        .map_err(|err| match err {
+            KernelError::InvalidArgument
+            | KernelError::MachSendInvalidDest
+            | KernelError::Terminated => {
+                SamplingError::ProcessTerminated("task_threads in get_thread_list", err)
+            }
+            err => SamplingError::Ignorable("task_threads in get_thread_list", err),
+        })?;
 
     let thread_acts =
         unsafe { std::slice::from_raw_parts(thread_list, thread_count as usize) }.to_owned();
@@ -214,7 +249,8 @@ fn get_thread_list(task: mach_port_t) -> kernel_error::Result<Vec<thread_act_t>>
             (thread_count as usize * mem::size_of::<thread_act_t>()) as mach_vm_size_t,
         )
     }
-    .into_result()?;
+    .into_result()
+    .map_err(|err| SamplingError::Fatal("mach_vm_deallocate in get_thread_list", err))?;
 
     Ok(thread_acts)
 }
